﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Battleship
{
    /// <summary>
    /// Класс, отвечающий за прорисовку и перемещение объекта-сущности
    /// </summary>
    public class DrawningBattleship
    {
        /// <summary>
        /// Класс-сущность
        /// </summary>
        public EntityBattleship Battleship { get; protected set; }
        /// <summary>
        /// Левая координата отрисовки боевого корабля
        /// </summary>
        protected float _startPosX;
        /// <summary>
        /// Верхняя кооридната отрисовки боевого корабля
        /// </summary>
        protected float _startPosY;
        /// <summary>
        /// Ширина окна отрисовки
        /// </summary>
        private int? _pictureWidth = null;
        /// <summary>
        /// Высота окна отрисовки
        /// </summary>
        private int? _pictureHeight = null;
        /// <summary>
        /// Ширина отрисовки боевого корабля
        /// </summary>
        private readonly int _battleshipWidth = 130;
        /// <summary>
        /// Высота отрисовки боевого корабля
        /// </summary>
        private readonly int _battleshipHeight = 40;
        /// <summary>
        /// Инициализация свойств
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес боевого корабля</param>
        /// <param name="bodyColor">Цвет корпуса</param>
        public DrawningBattleship(int speed, float weight, Color bodyColor)
        {
            Battleship = new EntityBattleship(speed, weight, bodyColor);
        }
        /// <summary>
        /// Инициализация свойств
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес военного корабля</param>
        /// <param name="bodyColor">Цвет корпуса корабля</param>
        /// <param name="battleshipWidth">Ширина отрисовки военного корабля</param>
        /// <param name="battleshipHeight">Высота отрисовки военного корабля</param>
        protected DrawningBattleship(int speed, float weight, Color bodyColor, int battleshipWidth, int battleshipHeight) :
            this(speed, weight, bodyColor)
        {
            _battleshipWidth = battleshipWidth;
            _battleshipHeight = battleshipHeight;
        }
        /// <summary>
        /// Установка позиции боевого корабля
        /// </summary>
        /// <param name="x">Координата X</param>
        /// <param name="y">Координата Y</param>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param>
        public void SetPosition(int x, int y, int width, int height)
        {
            // TODO checks
            if (x < 0 || y < 0 || width < x + _battleshipWidth || height < y + _battleshipHeight)
            {
                _pictureHeight = null;
                _pictureWidth = null;
                return;
            }
            _startPosX = x;
            _startPosY = y;
            _pictureWidth = width;
            _pictureHeight = height;
        }
        /// <summary>
        /// Изменение направления пермещения
        /// </summary>
        /// <param name="direction">Направление</param>
        public void MoveTransport(Direction direction)
        {
            if (!_pictureWidth.HasValue || !_pictureHeight.HasValue)
            {
                return;
            }
            switch (direction)
            {
                // вправо
                case Direction.Right:
                    if (_startPosX + _battleshipWidth + Battleship.Step < _pictureWidth)
                    {
                        _startPosX += Battleship.Step;
                    }
                    break;
                //влево
                case Direction.Left:
                    if (_startPosX - Battleship.Step > 0)
                    {
                        _startPosX -= Battleship.Step;
                    }
                    break;
                //вверх
                case Direction.Up:
                    if (_startPosY - Battleship.Step > 0)
                    {
                        _startPosY -= Battleship.Step;
                    }
                    break;
                //вниз
                case Direction.Down:
                    if (_startPosY + _battleshipHeight + Battleship.Step < _pictureHeight)
                    {
                        _startPosY += Battleship.Step;
                    }
                    break;
            }
        }
        /// <summary>
        /// Отрисовка боевого корабля
        /// </summary>
        /// <param name="g"></param>
        public virtual void DrawTransport(Graphics g)
        {
            if (_startPosX < 0 || _startPosY < 0
                || !_pictureHeight.HasValue || !_pictureWidth.HasValue)
            {
                return;
            }
            Pen pen = new(Color.Black);
            //границы боевого корабля
            g.DrawEllipse(pen, _startPosX + 60, _startPosY + 10, 10, 10);
            g.DrawRectangle(pen, _startPosX + 20, _startPosY + 10, 20, 10);
            g.DrawRectangle(pen, _startPosX - 4, _startPosY + 4, 4, 10);
            g.DrawRectangle(pen, _startPosX - 4, _startPosY + 16 , 4, 10);
            g.DrawRectangle(pen, _startPosX + 40, _startPosY + 4, 10, 20);
            g.DrawRectangle(pen, _startPosX , _startPosY, 110, 30);
            PointF point_triangle_1 = new PointF(_startPosX + 110, _startPosY);
            PointF point_triangle_2 = new PointF(_startPosX + 130, _startPosY + 15);
            PointF point_triangle_3 = new PointF(_startPosX + 110, _startPosY + 30);
            PointF[] assembled_triangle =
             {
                 point_triangle_1,
                 point_triangle_2,
                 point_triangle_3,
                 point_triangle_1
             };
            g.DrawPolygon(pen, assembled_triangle);

            //задние балки
            Brush brBlack = new SolidBrush(Color.Black);
            g.FillRectangle(brBlack, _startPosX -4, _startPosY + 4, 4, 10);
            g.FillRectangle(brBlack, _startPosX - 4, _startPosY + 16, 4, 10);

            //палуба корабля
            Brush br = new SolidBrush(Battleship?.BodyColor ?? Color.Black);
            g.FillRectangle(br, _startPosX, _startPosY, 110, 30);
            g.FillPolygon(br, assembled_triangle);

            //отсек корабля
            Brush brGray = new SolidBrush(Color.Gray);
            g.FillEllipse(brGray, _startPosX + 60, _startPosY + 10, 10, 10);
            g.FillRectangle(brGray, _startPosX + 20, _startPosY + 10, 20, 10);
            g.FillRectangle(brGray, _startPosX + 40, _startPosY + 4, 10, 20);

            //выделяем рамкой палубу
            g.DrawEllipse(pen, _startPosX + 60, _startPosY + 10, 10, 10);
            g.DrawRectangle(pen, _startPosX + 20, _startPosY + 10, 20, 10);
            g.DrawRectangle(pen, _startPosX + 40, _startPosY + 4, 10, 20);

        }
        /// <summary>
        /// Смена границ формы отрисовки
        /// </summary>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param>
        public void ChangeBorders(int width, int height)
        {
            _pictureWidth = width;
            _pictureHeight = height;
            if (_pictureWidth <= _battleshipWidth || _pictureHeight <= _battleshipHeight)
            {
                _pictureWidth = null;
                _pictureHeight = null;
                return;
            }
            if (_startPosX + _battleshipWidth > _pictureWidth)
            {
                _startPosX = _pictureWidth.Value - _battleshipWidth;
            }
            if (_startPosY + _battleshipHeight > _pictureHeight)
            {
                _startPosY = _pictureHeight.Value - _battleshipHeight;
            }
        }
        /// <summary>
        /// Получение текущей позиции объекта
        /// </summary>
        /// <returns></returns>
        public (float Left, float Right, float Top, float Bottom) GetCurrentPosition()
        {
            return (_startPosX, _startPosX + _battleshipWidth, _startPosY, _startPosY + _battleshipHeight);
        }
    }
}
