﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Battleship
{
    /// <summary>
    /// Параметризованный набор объектов
    /// </summary>
    /// <typeparam name="T"></typeparam>
    internal class SetBattleshipGeneric<T>
        where T : class
    {
        private readonly List<T> _places;

        public int Count => _places.Count;
        private int BattleshipPlaces = 0;

        private readonly int _maxCount;

        public SetBattleshipGeneric(int count)
        {
            _maxCount = count;
            _places = new List<T>();
        }

        public int Insert(T battleship)
        {
            if (_places.Count + 1 >= _maxCount)
                return -1;
            _places.Insert(0, battleship);
            return 0;
        }

        public int Insert(T battleship, int position)
        {
            if (position >= _maxCount || position < 0)
                return -1;
            if (_places.Count + 1 >= _maxCount)
                return -1;
            _places.Insert(position, battleship);
            return position;
        }

        public T Remove(int position)
        {
            if (position < 0 || position >= _maxCount) return null;
            T savedBattleship = _places[position];
            _places.RemoveAt(position);
            return savedBattleship;
        }

        public T this[int position]
        {
            get
            {
                if (position < 0 || position >= _maxCount) return null;
                return _places[position];
            }
            set
            {
                Insert(value, position);
            }

        }

        public IEnumerable<T> GetBattleship()
        {
            foreach (var battleship in _places)
            {
                if (battleship != null)
                {
                    yield return battleship;
                }
                else
                {
                    yield break;
                }
            }
        }
    }
}
